from rust_reversi import Arena
import sys
import os
import pytest

N_GAMES = 1000

RANDOM_PLAYER = "players/random_player.py"
NONEXISTENT_PLAYER = "players/nonexistent_player.py"
SLOW_PLAYER = "players/slow_player.py"


def get_player_path(filename: str) -> str:
    return os.path.join(os.path.dirname(__file__), filename)


def test_random_vs_random():
    python = sys.executable
    random_player = get_player_path(RANDOM_PLAYER)
    arena = Arena([python, random_player], [python, random_player])
    arena.play_n(N_GAMES)
    wins1, wins2, draws = arena.get_stats()
    pieces1, pieces2 = arena.get_pieces()

    assert wins1 + wins2 + draws == N_GAMES
    assert pieces1 + pieces2 > 0
    win_ratio = abs((wins1 - wins2) / N_GAMES)
    assert win_ratio < 0.1  # sometimes it fails


def test_arena_odd_games():
    python = sys.executable
    random_player = get_player_path(RANDOM_PLAYER)
    arena = Arena([python, random_player], [python, random_player])

    with pytest.raises(ValueError, match="Game count must be even"):
        arena.play_n(999)


def test_arena_invalid_player():
    python = sys.executable
    invalid_player = get_player_path(NONEXISTENT_PLAYER)
    arena = Arena([python, invalid_player], [python, invalid_player])

    with pytest.raises(ValueError, match="Engine start error"):
        arena.play_n(2)


def test_arena_multiple_sessions():
    python = sys.executable
    random_player = get_player_path(RANDOM_PLAYER)
    arena = Arena([python, random_player], [python, random_player])

    arena.play_n(100)
    first_stats = arena.get_stats()

    arena.play_n(100)
    second_stats = arena.get_stats()

    assert sum(second_stats) == 200
    assert all(b >= a for a, b in zip(first_stats, second_stats))


def test_arena_timeout():
    python = sys.executable
    slow_player = get_player_path(SLOW_PLAYER)
    random_player = get_player_path(RANDOM_PLAYER)

    arena = Arena([python, slow_player], [python, random_player])

    with pytest.raises(ValueError, match="Game error: (Black|White)Timeout"):
        arena.play_n(2)



from typing import List, Optional, Tuple

class Turn:
    BLACK: "Turn"
    WHITE: "Turn"

    def __eq__(self, other: object) -> bool: ...
    def __str__(self) -> str: ...

class Color:
    EMPTY: "Color"
    BLACK: "Color"
    WHITE: "Color"

    def __eq__(self, other: object) -> bool: ...
    def __str__(self) -> str: ...

class Board:
    def __init__(self) -> None: ...
    def get_board(self) -> Tuple[int, int, Turn]: ...
    def set_board(self, player_board: int, opponent_board: int, turn: Turn) -> None: ...
    def set_board_str(self, line: str, turn: Turn) -> None: ...
    def get_board_line(self) -> str: ...
    def get_board_vec_black(self) -> List[Color]: ...
    def get_board_vec_turn(self) -> List[Color]: ...
    def get_board_matrix(self) -> List[List[List[int]]]: ...
    def player_piece_num(self) -> int: ...
    def opponent_piece_num(self) -> int: ...
    def black_piece_num(self) -> int: ...
    def white_piece_num(self) -> int: ...
    def piece_sum(self) -> int: ...
    def diff_piece_num(self) -> int: ...
    def get_legal_moves(self) -> int: ...
    def get_legal_moves_vec(self) -> List[int]: ...
    def get_legal_moves_tf(self) -> List[bool]: ...
    def is_legal_move(self, pos: int) -> bool: ...
    def get_child_boards(self) -> List["Board"]: ...
    def do_move(self, pos: int) -> None: ...
    def do_pass(self) -> None: ...
    def is_pass(self) -> bool: ...
    def is_game_over(self) -> bool: ...
    def is_win(self) -> bool: ...
    def is_lose(self) -> bool: ...
    def is_draw(self) -> bool: ...
    def is_black_win(self) -> bool: ...
    def is_white_win(self) -> bool: ...
    def get_winner(self) -> Optional[Turn]: ...
    def get_random_move(self) -> int: ...
    def __str__(self) -> str: ...
    def clone(self) -> "Board": ...

class Arena:
    def __init__(self, command1: List[str], command2: List[str]) -> None: ...
    def play_n(self, n: int) -> None: ...
    def get_stats(self) -> Tuple[int, int, int]: ...
    def get_pieces(self) -> Tuple[int, int]: ...
